/* this is an example to use the matlab engine. this means calling */
/* matlab from within the C program. 
 * this file is the c-part of the specman to matlab interface.
 * it includes the implementation of the low-level c routines that communicate
 * with the matlab engine.
 *
 * Usage: 
 * 		for usage info please see the demo, demo.ecom and hopefully the readme.txt.
 * Compilation:	
 * 		this file has to be compiled and linked with specman.
 * 		the makefile takes care of that. 
 * 		just issue 'make' at the shell prompt.
 *
 * 		in order for the compilation to work properly make sure that:
 *
 * 		$MATLAB
 * 			points to the matlab installation directory.
 * 		
 * 		$LD_LIBRARY_PATH 
 * 			points to the matlab libraries:
 * 				$MATLAB/sys/os/glnx86:$MATLAB/extern/lib/glnx86
 * 		if the linker still complains that it can't find the -lmat etc.,
 * 		then add the -L$MATLAB/extern/lib/<arch> to the compilation script.
 * 		on linux that would be:
 * 			-ldflags "-L$MATLAB/extern/lib/glnx86"
 *
 * 		the matlab binaries should be in the $PATH.
 *
 * Written by:
 * 		meir@verisity.com 
 * 		based on a previous interface provided by umer@verisity.com.
 * 
 * Types: 
   	typedef union { 
  	    unsigned int uint;
  		unsigned int lh[2];
  		double       d;      
  		long long	uint64;
  	} sn_mb_double_t;
 *	this type is used for conversion of double and 64bits data types from/to
 *	the matlab ebngine and to/from specman data types.
 *
  	struct matlab_sn_s { 
  		char*   out;
  		Engine* ep ;		//the pointer to the matlab engine as retrieved from matlab.
  		mxArray*  mxrp;     //a real uint vector to be used for puting.
  		mxArray*  mxcp;     //a complex uint vector to be used for puting.
  		mxArray*  mxrdp;        //a real double vector to be used for puting.
  		mxArray*  mxcdp;        //a complex double vector to be used for puting.
  		matlab_sn_s() : ep(0) {};
  	};
 *	this struct is used to hold the matlab engine connection parameters.
 *	in addition a banch of arrays are allocated at the initialisation in order to 
 *	avoid their allocation each time that data is sent from specman to matlab.
 *  		
 * Static variables:
 *
 * 	static struct matlab_sn_s mbsn[1];
 * 		holds the matlab engine connection parameters.
 * 		it is defined as static in order to avoid reconnections after restore
 * 		is issued in specman.
 *	static char	matlab_output[MATLAB_OUT_BUFFER_SZ];
 *		a buffer to holds output info from matlab. it is 1k in size.
 *	static char sn_matlab_err_not_running[];
 *		 general error msg.
 *
 * Functions: 
	SN_TYPE(matlab_rc_t) sn_matlab_init(t__matlab_ifc_s ifc)
		Description:
			initialise the sn/matlab interface.
			1. check if a connection has already been established.
			2. if not try to establish one and save its engine pointer into mbsn->ep.
			3. allocate a banch of matlab array to be used for data transfer from sn to mb.
			4. register sn_matlab_finish()  to be called at exit.
				hence, it will be called when sn exits.
		Called by (e methods):
			matlab.do_init()
			
	void sn_matlab_chk_err(SN_TYPE(matlab_ifc_s) ifc, int rc) 
		Description:
			- checks the status of the last matlab library call.
			- sets the status accordingly in the matlab global e struct.
		Called by (e methods):
			other c functions in this module after each call to a matlab library 
			function.
		
	SN_TYPE(matlab_rc_t) sn_matlab_exec(t__matlab_ifc_s ifc, SN_TYPE(string) line) {
		Description:
			- send a matlab command to the matlab engine to be executed.
		Called by (e methods):
			matlab.exec()

	SN_TYPE(matlab_rc_t) sn_matlab_get_var(t__matlab_ifc_s ifc, SN_TYPE(sn_matlab_var_s) var) 
		Description:
			transfer data from matlab to sn.
			the matlab variable from which data is transferred is designated by:
				'var.name'.
			only the pointer to the retrieved data is saved.
			this is principally enough for data access (read only).
			the data can then be accessed using the get_item() methods of matlab_mx_array_s.
		Called by (e methods):
			matlab.get_var(v : sn_matlab_var_s) : matlab_rc_t 
	
	SN_TYPE(matlab_rc_t) sn_matlab_put_var(t__matlab_ifc_s ifc, SN_TYPE(sn_matlab_var_s) var) 
		Description:
			takes the data carried by 'var' and transfers it to matlab.
			the data will be put into a matlab variable, which is designated by the 'var.name'
			string.
		Called by (e methods):
			matlab.put_var(v : sn_matlab_var_s) : matlab_rc_t 

	SN_TYPE(bool)  sn_matlab_is_equal( 
			t__matlab_ifc_s ifc, 
			SN_TYPE(sn_matlab_var_s) sn_var, 
			SN_TYPE(matlab_mx_array_s) mx_arr )  
		Description:
			compares the data of two variables and return
			TRUE	when both are equal
			FALSE	when the arrays differ.
			the first variable is designated by an sn_matlab_var_s, which means that its data
			is saved in a 'e list'.
			the second variable is a mx_array, whose data was retrieved from matlab by a previous
			get_var() call.
		Called by (e methods):
			matlab.is_equal(sn_var : sn_matlab_var_s, mb_var : matlab_mx_array_s) : bool

	SN_TYPE(bool)  sn_matlab_is_equal_mx( 
			t__matlab_ifc_s ifc, 
			SN_TYPE(matlab_mx_array_s) mx1, 
			SN_TYPE(matlab_mx_array_s) mx2) 
		Description:
			compares two mx_arrays.
			the data of both array is assumed to have been previously retrieved from matlab.
		Called by (e methods):
			matlab.is_equal_mx(mx1 : matlab_mx_array_s, mx2 : matlab_mx_array_s) : bool 

	SN_TYPE(bool)  sn_matlab_mx_to_sn_var( 
			t__matlab_ifc_s ifc, 
			SN_TYPE(matlab_mx_array_s) mx, 
			SN_TYPE(sn_matlab_var_s) snvar)  
		Description:
			convert the data that was retrieved from matlab by a previous get_var
			to an 'e list'.
			this step is principally not necessary and should be avoided 
			since it might involved copying of large amounts of data from one place to the other.
			better is to use the mx_array directly by accessing its elements using the 
			get_item() method.
		Called by (e methods):
			matlab.mx_to_sn(mx : matlab_mx_array_s) : sn_matlab_var_s

	void sn_matlab_get_mx_item(
			SN_TYPE(matlab_mx_array_s) arr, 
			SN_TYPE(uint) ro, SN_TYPE(uint) co, 
			SN_TYPE(sn_matlab_complex_s) res) 
		Description:
			returns the data associated with an element of the matrix designated 
			by the arr->mx_ptr pointer.
			this pointer was received from matlab durin a previous get_var() call.
		Parameters:
			arr
				designated a previously returned matlab_mx_array_s that was 
				allocated to hold retrieved data from matlab.
			ro
				the row number of the element to be accessed.
			co
				the column number of the element to be accessed.
				note that counting starts with 0.
				hence the 1st element is (ro,co) == (0,0)
				and the last one is:  (arr->rows-1, arr->cols-1)
		Called by (e methods):
			matlab_mx_array_s.get_item(ro : uint, co : uint ) : sn_matlab_complex_s
		
*/
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>   
#include <malloc.h>
#include <string.h>

#define BUFSIZE 256
  /************************************************************/
  /* step 1: include this file and link the proper libraries in the makefile */
  /************************************************************/
#include "engine.h"
#include "sn_matlab.h"

extern "C"  void sn_matlab_finish(void);
extern "C"  void sn_dout(SN_STRING);

#define MATLAB_OUT_BUFFER_SZ 1024 
typedef union { 
	    unsigned int uint;
		unsigned int lh[2];
		double       d;      
		long long	uint64;
} sn_mb_double_t;
struct matlab_sn_s { 
	char*   out;
	Engine* ep ;
	mxArray*  mxrp;     //a real uint vector to be used for puting.
	mxArray*  mxcp;     //a complex uint vector to be used for puting.
	mxArray*  mxrdp;        //a real double vector to be used for puting.
	mxArray*  mxcdp;        //a complex double vector to be used for puting.
	matlab_sn_s() : ep(0) {};
};
static struct matlab_sn_s mbsn[1];
static char	matlab_output[MATLAB_OUT_BUFFER_SZ];
static char sn_matlab_err_not_running[] = 
	"Matlab session no longer running!"; 

void set_mx_ptr( s__main__matlab_mx_array_s * arr, unsigned long mxarray) {
  arr->mx_ptr = (unsigned long) mxarray;
  arr->mx_ptr_upper = mxarray>>32;
};

unsigned long get_mx_ptr( s__main__matlab_mx_array_s *arr) {
  unsigned long temp = 0;
  temp = arr->mx_ptr_upper;
  temp = temp << 32;
  temp += arr->mx_ptr ;
  return temp;
};


void sn_matlab_chk_err(SN_TYPE(matlab_ifc_s) ifc, int rc);
//
#define DBG_OUT(_flag, _string) {if ((_flag)) sn_dout(_string);}
//
//if not already connected then get a connection and register a disconnecting function atexit
SN_TYPE(matlab_rc_t) sn_matlab_init(t__matlab_ifc_s ifc) {
	/************************************************************/
	/* step 1: initialize the engine pointer and the data transfer area  */
	/* to be used inside of matlab*/
	/************************************************************/
	Engine *ep;	//a pointer to the matlab engine. will be received from matlab.
	int rc = 0;
//	int dims[] = {1,1};
	const mwSize dims[] = {1,1};

	mbsn->mxrp = 0; mbsn->mxcp = 0; mbsn->mxrdp = 0; mbsn->mxcdp = 0;
	/************************************************************/
	/*  step 3: wake up matlab by invoking the following command */
	/* note that the command expects that UNIX>matlab call works fine */
	/************************************************************/
	ifc->eng = 0; 
	ifc->last_rc = matlab_rc_t__SUCCESS;
	ifc->state = SN_DEFINE(MATLAB_STATE_UNINIT);



        /****************************Start of engDemo.c***********************************/
	 mxArray *T = NULL, *result = NULL;
        char buffer[BUFSIZE+1];
        double time[10] = { 0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0 };

        /*
         * Start the MATLAB engine locally by executing the string
         * "matlab"
         *
         * To start the session on a remote host, use the name of
         * the host as the string rather than \0
         *
         * For more complicated cases, use any string with whitespace,
         * and that string will be executed literally to start MATLAB
         */
        if (!(ep = engOpen("\0"))) {
                fprintf(stderr, "\nCan't start MATLAB engine\n");
		return SN_ENUM(matlab_rc_t, ERROR);
        }


        printf("After engOpen engdemo ep = %d.\n",ep);
//        /*
//         * PART I
//         *
//         * For the first half of this demonstration, we will send data
//         * to MATLAB, analyze the data, and plot the result.
//         */
//
//        /*
//         * Create a variable for our data
//         */
//        T = mxCreateDoubleMatrix(1, 10, mxREAL);
//        memcpy((void *)mxGetPr(T), (void *)time, sizeof(time));
//        /*
//         * Place the variable T into the MATLAB workspace
//         */
//        engPutVariable(ep, "T", T);
//
//        /*
//         * Evaluate a function of time, distance = (1/2)g.*t.^2
//         * (g is the acceleration due to gravity)
//         */
//        engEvalString(ep, "D = .5.*(-9.8).*T.^2;");
//
//        /*
//         * Plot the result
//         */
//        engEvalString(ep, "plot(T,D);");
//        engEvalString(ep, "title('Position vs. Time for a falling object');");
//	engEvalString(ep, "xlabel('Time (seconds)');");
//        engEvalString(ep, "ylabel('Position (meters)');");
//
//        /*
//         * use fgetc() to make sure that we pause long enough to be
//         * able to see the plot
//         */
//        printf("Hit return to continue\n\n");
//        fgetc(stdin);
//        /*
//         * We're done for Part I! Free memory, close MATLAB engine.
//         */
//        printf("Done for Part I.\n");
//        mxDestroyArray(T);
//        engEvalString(ep, "close;");
//
//
//        /*
//         * PART II
//         *
//         * For the second half of this demonstration, we will request
//         * a MATLAB string, which should define a variable X.  MATLAB
//         * will evaluate the string and create the variable.  We
//         * will then recover the variable, and determine its type.
//         */
//
//        /*
//         * Use engOutputBuffer to capture MATLAB output, so we can
//         * echo it back.  Ensure first that the buffer is always NULL
//         * terminated.
//         */
//
//        buffer[BUFSIZE] = '\0';
//        engOutputBuffer(ep, buffer, BUFSIZE);
//
//	while (result == NULL) {
//            char str[BUFSIZE+1];
//            /*
//             * Get a string input from the user
//             */
//            printf("Enter a MATLAB command to evaluate.  This command should\n");
//            printf("create a variable X.  This program will then determine\n");
//            printf("what kind of variable you created.\n");
//            printf("For example: X = 1:5\n");
//            printf(">> ");
//
//            fgets(str, BUFSIZE, stdin);
//
//            /*
//             * Evaluate input with engEvalString
//             */
//            engEvalString(ep, str);
//
//            /*
//             * Echo the output from the command.  First two characters are
//             * always the double prompt (>>).
//             */
//            printf("%s", buffer+2);
//
//            /*
//             * Get result of computation
//             */
//            printf("\nRetrieving X...\n");
//            if ((result = engGetVariable(ep,"X")) == NULL)
//              printf("Oops! You didn't create a variable X.\n\n");
//            else {
//                printf("X is class %s\t\n", mxGetClassName(result));
//            }
//        }
//
//        /*
//         * We're done! Free memory, close MATLAB engine and exit.
//         */
//        printf("Done!\n");
 //       mxDestroyArray(result);
       // engClose(ep);


        /*****************************end of engDemo.c**********************************/	


	printf("***** sn_matlab_init: mbsn->ep=%x", mbsn->ep);
	//check if we have already saved a valid pointer to matlab.
	if(mbsn->ep == 0) {
		//get a pointer if we don't have one.
//		printf("\n***** connecting to the matlab engine. please wait...*****\n");
//		if (!(ep = engOpen("\0"))) { 
//			return SN_ENUM(matlab_rc_t, ERROR);
//		}
	        printf("***** sn_matlab_init: ep=%x", ep);
		DBG_OUT(ifc->cdbg_flag, "sn_matlab_init():\n\t got a valid pointer." );
		//save the pointer to the matlab environment
		ifc->eng = ep;
		mbsn->ep = ep;
		//register sn_matlab_finish to be executed when we exit specman.
		if( atexit(sn_matlab_finish) != 0 ) {
			return SN_ENUM(matlab_rc_t, ERROR);
		};
	} else {
		//chk validity of mbsn->ep and assign it to ifc->ep
		ifc->eng = mbsn->ep;
		ep = mbsn->ep;
	};
	ifc->state = SN_DEFINE(MATLAB_STATE_INIT);
		//set an output buffer to capture matlab output
	mbsn->out = matlab_output;
		//clear the buffer in case of a restore 
	matlab_output[0] = 0;
	rc = engOutputBuffer( ep, matlab_output, MATLAB_OUT_BUFFER_SZ ); 
	DBG_OUT(ifc->cdbg_flag, "sn_matlab_init():\n\t set output buffer." );
	sn_matlab_chk_err(ifc, rc);
		//alloc space for a tmp mxArray. to be used for putting vars.
	mbsn->mxcp = mxCreateNumericArray(2, dims, mxINT32_CLASS, mxCOMPLEX);
	//ifc->mxc->mx_ptr = (unsigned long) mbsn->mxcp;	// set the pointer in specman matlab struct
        set_mx_ptr(ifc->mxc,(unsigned long) mbsn->mxcp);	// set the pointer in specman matlab struct
	//	ifc->mxc->mx_ptr = (unsigned int) mbsn->mxcp;	// set the pointer in specman matlab struct
		//real uint vector
	mbsn->mxrp = mxCreateNumericArray(2, dims, mxINT32_CLASS, mxREAL);
        set_mx_ptr(ifc->mxr,(unsigned long) mbsn->mxrp);	// set the pointer in specman matlab struct
	//ifc->mxr->mx_ptr = (unsigned long) mbsn->mxrp;	// set the pointer in specman matlab struct
	//ifc->mxr->mx_ptr = (unsigned int) mbsn->mxrp;	// set the pointer in specman matlab struct
		//real double vector
	mbsn->mxrdp = mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxREAL);
        set_mx_ptr(ifc->mxrd,(unsigned long) mbsn->mxrdp);	// set the pointer in specman matlab struct
	//ifc->mxrd->mx_ptr = (unsigned long) mbsn->mxrdp;	// set the pointer in specman matlab struct
	//ifc->mxrd->mx_ptr = (unsigned int) mbsn->mxrdp;	// set the pointer in specman matlab struct
		//complex double vector
	mbsn->mxcdp = mxCreateNumericArray(2, dims, mxDOUBLE_CLASS, mxCOMPLEX);
        set_mx_ptr(ifc->mxcd,(unsigned long) mbsn->mxcdp);	// set the pointer in specman matlab struct
	//	ifc->mxcd->mx_ptr = (unsigned long) mbsn->mxcdp;	// set the pointer in specman matlab struct
	//ifc->mxcd->mx_ptr = (unsigned int) mbsn->mxcdp;	// set the pointer in specman matlab struct
		//check successful alloc
	if( (mbsn->mxrp == 0) || (mbsn->mxcp ==0) || 
	    (mbsn->mxrdp == 0) || (mbsn->mxcdp == 0) ) {
		DBG_OUT(ifc->cdbg_flag, "sn_matlab_init():\n\t failure in allocating mxArrays!!!");
		return SN_ENUM(matlab_rc_t, ERROR);
	};
	return ifc->last_rc;
}
void sn_matlab_chk_err(SN_TYPE(matlab_ifc_s) ifc, int rc) {
	if( rc != 0 ) {
		ifc->last_rc = SN_ENUM(matlab_rc_t, ERROR);
		ifc->last_msg = sn_matlab_err_not_running; 
	} else { 
		ifc->last_rc = SN_ENUM(matlab_rc_t, SUCCESS);
		ifc->last_msg = matlab_output;
	}
	DBG_OUT(ifc->cdbg_flag, "output from matlab: \n");
	DBG_OUT(ifc->cdbg_flag, matlab_output);
}
//send commands for execution to the matlab engine.
SN_TYPE(matlab_rc_t) sn_matlab_exec(t__matlab_ifc_s ifc, SN_TYPE(string) line) {
	Engine* ep = (Engine*) ifc->eng;	//get the engine pointer 
	int rc = 0;
		//send line to matlab for execution
	DBG_OUT(ifc->cdbg_flag, "sn_matlab_exec():\n\t");
	DBG_OUT(ifc->cdbg_flag, line);	
	matlab_output[0] = 0;
		//send to matlab.
	rc = engEvalString(ep, line) ;
	sn_matlab_chk_err(ifc, rc);
	return ifc->last_rc;
}
//sn_matlab_finish will be executed when specman exits.
//hence, it is planed to be executed only once, so that specman restore's
//don't break the connection to matlab.
void sn_matlab_finish(void) {
	int rc =0;
	printf("***** sn_matlab_finish: mbsn->ep=%x", mbsn->ep);
	if( mbsn->ep ) {
		rc = engClose(mbsn->ep) ;
		mbsn->ep = 0;  
	};
}
SN_TYPE(matlab_rc_t) sn_matlab_fini(t__matlab_ifc_s ifc) {
	ifc->eng = 0; 
	return ifc->last_rc;
}
//retrieve a variable from the matlab workspace.
//the variable is designated by its name in the matlab workspace.
//variables in matlab are usually stored in matrices where each element is 
//either a double precision real or complex number.
//these valus will be transferred and can be accessed directly by using the get_item
//method or they can be converted first to specman lists, flattening the matrix thereby.
SN_TYPE(matlab_rc_t) sn_matlab_get_var(t__matlab_ifc_s ifc, SN_TYPE(sn_matlab_var_s) var) {
	Engine* ep = (Engine*) ifc->eng;  
	int rc = 0;
	mxArray* mxp = 0;	//results from matlab are returned as mxArray struct.
	mxClassID class_id;
		//
	if(ifc->cdbg_flag) printf("\nD> sn_matlab_get_var(%s):", var->name);
	mxp = engGetVariable(ep, var->name) ;		//get the mxArray 
		//check successful completion
	if( mxp == NULL ) {
		rc = 0xF; 
		DBG_OUT(ifc->cdbg_flag, "\n\t failed to get the variable data");
	} else {
		DBG_OUT(ifc->cdbg_flag, "\n\t got the variable data successfuly!");
	}
	sn_matlab_chk_err(ifc, rc);
		//set a reference in our e env 

	if(ifc->cdbg_flag) printf("\nD> mxp =%p :", mxp);
	//	var->mx_arr->mx_ptr = (unsigned long) mxp;					//set external pointer in e
	set_mx_ptr(var->mx_arr, (unsigned long) mxp);					//set external pointer in e
		//set class
	class_id = mxGetClassID(mxp);
		//set the elements class id.

	switch( class_id ) {
		case mxDOUBLE_CLASS : 
			var->its_class = matlab_class_id_t__MB_DOUBLE;
			var->mx_arr->its_class = SN_ENUM(matlab_class_id_t, MB_DOUBLE);
			break;
		case mxINT32_CLASS:
		case mxUINT32_CLASS:
			var->its_class = SN_ENUM(matlab_class_id_t, MB_INT32);
			var->mx_arr->its_class = SN_ENUM(matlab_class_id_t, MB_INT32);
			break;
		case mxINT64_CLASS:
		case mxUINT64_CLASS:
			var->its_class = SN_ENUM(matlab_class_id_t, MB_INT64);
			var->mx_arr->its_class = SN_ENUM(matlab_class_id_t, MB_INT64);
			break;
		default :
			sn_out("Error! sn_matlab_get_var(): class id returned from matlab is not supported yet!");
			break;
	}
		//set complexity
	if( mxIsComplex(mxp) ) {
		var->is_complex = TRUE;
		var->mx_arr->is_complex = TRUE;
	} else {
		var->is_complex = FALSE;
		var->mx_arr->is_complex = FALSE;
	};
		//set number of rows
	rc = mxGetM( mxp );
	var->mx_arr->rows = rc;
		//set number of columns
	rc = mxGetN( mxp );
	var->mx_arr->cols = rc;
		//
	return ifc->last_rc;
}
SN_TYPE(matlab_rc_t) sn_matlab_put_var(t__matlab_ifc_s ifc, SN_TYPE(sn_matlab_var_s) var) {
	Engine* ep = (Engine*) ifc->eng;  
	int rc = 0;
	mxArray* mxp = 0;
	int no_of_el = 0;
		//
	DBG_OUT(ifc->cdbg_flag, "sn_matlab_put_var():\n\t");
		//choose an existing ptr according to 'var' type
		//first check if it is a uint or double array 
	no_of_el = LIST_SIZE(var->r_data);
	if( var->its_class == SN_ENUM(matlab_class_id_t, MB_INT32) ) { //uint array
		if( var->is_complex == FALSE ) //real uint array
			mxp = mbsn->mxrp;
		else 
			mxp = mbsn->mxcp;
	} else {	//double array
		no_of_el >>= 1;//double elements are double in size...
		if( var->is_complex == FALSE ) //real double array
			mxp = mbsn->mxrdp;
		else 
			mxp = mbsn->mxcdp;
	};
		//set length of vector
	mxSetM(mxp, no_of_el );	//#of rows
	mxSetN(mxp, 1 );	//#of cols == 1 always, when putting back data. 
						//matlab always calculate with column vectors hence the above.
		//set data pointers
	mxSetPr(mxp, (double*) (var->r_data->data));
	if( var->is_complex ) 
		mxSetPi(mxp, (double*) (var->i_data->data));
		//put variable back to matlab
	rc = engPutVariable(ep, var->name, mxp);
	sn_matlab_chk_err(ifc, rc);
		//
	return ifc->last_rc;
}
SN_TYPE(bool) sn_matlab_is_double_mx(SN_TYPE(matlab_mx_array_s) arr) {
  //	return ( (mxGetClassID( (mxArray*) arr->mx_ptr ) == mxDOUBLE_CLASS) ? TRUE : FALSE ) ;
  return ( (mxGetClassID( (mxArray*) get_mx_ptr(arr) ) == mxDOUBLE_CLASS) ? TRUE : FALSE ) ;
}
SN_TYPE(bool) sn_matlab_is_complex_mx(SN_TYPE(matlab_mx_array_s) arr) {
  //	return ( mxIsComplex( (mxArray*) arr->mx_ptr ) ? TRUE : FALSE ) ;
  return ( mxIsComplex( (mxArray*) get_mx_ptr(arr) ) ? TRUE : FALSE ) ;
}
SN_TYPE(bool)  sn_matlab_is_equal( 
		t__matlab_ifc_s ifc, SN_TYPE(sn_matlab_var_s) sn_var, SN_TYPE(matlab_mx_array_s) mb_var) {  
	int rc = 0;
	//	mxArray* mxp = (mxArray*) mb_var->mx_ptr;
	mxArray* mxp = (mxArray*) get_mx_ptr(mb_var);
	void* s1 = (void*) sn_var->r_data->data;	//get the sn real list data pointer
	void* s2 = (void*) mxGetPr( mxp );			//get the mx real vector data pointer
	int   no_of_bytes = LIST_SIZE( sn_var->r_data ) * sizeof( t__uint );//t_uint take 4 bytes
		//compare memories
	rc = memcmp(s1, s2, no_of_bytes);
		//if complex cmp also the imag array
	if( (rc == 0) && (sn_var->is_complex) ) {
		s1 = (void*) sn_var->i_data->data;
		s2 = (void*) mxGetPi( mxp );
		rc = memcmp(s1, s2, no_of_bytes);
	};
	if( rc != 0 ) return FALSE;
	else 		  return TRUE;
}
SN_TYPE(bool)  sn_matlab_is_equal_mx( 
		t__matlab_ifc_s ifc, SN_TYPE(matlab_mx_array_s) mx1, SN_TYPE(matlab_mx_array_s) mx2) 
{  
	int rc = 0;
	//	mxArray* mxp1 = (mxArray*) mx1->mx_ptr;
	mxArray* mxp1 = (mxArray*) get_mx_ptr(mx1);
	//      mxArray* mxp2 = (mxArray*) mx2->mx_ptr;
	mxArray* mxp2 = (mxArray*) get_mx_ptr(mx2);
	void* s1 = (void*) mxGetPr( mxp1 );			//get the mx real vector data pointer
	void* s2 = (void*) mxGetPr( mxp2 );			//get the mx real vector data pointer
	int   no_of_bytes = mx1->cols * mx2->rows * sizeof( t__uint );//t_uint take 4 bytes
	if( mx1->its_class == SN_ENUM(matlab_class_id_t, MB_DOUBLE) ) no_of_bytes <<= 1; 
		//compare memories
	rc = memcmp(s1, s2, no_of_bytes);
		//if complex cmp also the imag array
	if( (rc == 0) && (mx1->is_complex) ) {
		s1 = (void*) mxGetPi( mxp1 );
		s2 = (void*) mxGetPi( mxp2 );
		rc = memcmp(s1, s2, no_of_bytes);
	};
	if( rc != 0 ) return FALSE;
	else 		  return TRUE;
};
SN_TYPE(bool)  sn_matlab_mx_to_sn_var( 
		t__matlab_ifc_s ifc, SN_TYPE(matlab_mx_array_s) mx, SN_TYPE(sn_matlab_var_s) snvar) {  
	void* rc = 0;
	//	mxArray* mxp = (mxArray*) mx->mx_ptr;
	mxArray* mxp = (mxArray*) get_mx_ptr(mx);
	void* dest = (void*) snvar->r_data->data;	//get the sn real list data pointer
	void* src = (void*) mxGetPr( mxp );			//get the mx real vector data pointer
	int   no_of_bytes = LIST_SIZE( snvar->r_data ) * sizeof( t__uint );//t_uint take 4 bytes
		//cp memories
	rc = memcpy(dest, src, no_of_bytes);
		//if complex cmp also the imag array
	if( (rc == dest) && (snvar->is_complex) ) {
		dest = (void*) snvar->i_data->data;
		src = (void*) mxGetPi( mxp );
		rc = memcpy(dest, src, no_of_bytes);
	};
	if( rc != dest ) return FALSE;
	else 		  	 return TRUE;
};
void sn_matlab_get_mx_item(SN_TYPE(matlab_mx_array_s) arr, 
							SN_TYPE(uint) ro, SN_TYPE(uint) co, SN_TYPE(sn_matlab_complex_s) res) {
  //	mxArray* mxp = (mxArray*) arr->mx_ptr;
        mxArray* mxp = (mxArray*) get_mx_ptr(arr);
	//	printf("\nD> mxp =%p :", mxp);
	sn_mb_double_t  mb2sn[1];
	mxClassID class_id;
	unsigned int idx = 0;
	
		//first set the info accordingly
	if( mxIsComplex(mxp) ) {
		res->is_complex = TRUE;
	} else {
		res->is_complex = FALSE;
	};
		//check if double
	class_id = mxGetClassID(mxp);
	switch( class_id ) {
		case mxDOUBLE_CLASS : 
			res->is_double = TRUE;
			res->is_int64  = FALSE; 
			break;
		case mxINT64_CLASS:
		case mxUINT64_CLASS:
			res->is_double = FALSE;
			res->is_int64  = TRUE;
			break;
		default :
			res->is_double = FALSE;
			res->is_int64  = FALSE;
	}
		//get pointers to real data and the item at index=idx
		//calculate the index out of the row,col info
		//counting starts at [0,0], i.e., the 1st element in the array
		//[1,0] is the second element in the array 
	idx = (co * arr->rows) + ro;
	if( res->is_double ) {
		double* dp = (double*) mxGetPr( mxp ); 
			//acess the item at index idx
		mb2sn->d = *(dp + idx);
		res->r->is_double = TRUE;
		res->r->low = mb2sn->lh[0];
		res->r->high = mb2sn->lh[1];
		if( res->is_complex ) {
			dp = (double*) mxGetPi( mxp ); 
				//acess the item at index idx
			mb2sn->d = *(dp + idx);
			res->i->is_double = TRUE;
			res->i->low = mb2sn->lh[0];
			res->i->high = mb2sn->lh[1];
		};
	} else if( res->is_int64 ) {
		long long* dp = (long long*) mxGetPr( mxp ); 
			//acess the item at index idx
		mb2sn->uint64 = *(dp + idx);
		res->r->is_double = FALSE;
		res->r->is_int64 = TRUE;
		res->r->low = mb2sn->lh[0];
		res->r->high = mb2sn->lh[1];
		if( res->is_complex ) {
			dp = (long long*) mxGetPi( mxp ); 
				//acess the item at index idx
			mb2sn->d = *(dp + idx);
			res->i->is_double = FALSE;
			res->i->is_int64 = TRUE;
			res->i->low = mb2sn->lh[0];
			res->i->high = mb2sn->lh[1];
		};
	} else {//uint array
		t__uint* uip = (t__uint*) mxGetPr( mxp ); 
			//acess the item at index idx
		mb2sn->uint = *(uip + idx);
		res->r->is_double = FALSE;
		res->r->low = mb2sn->uint;
		res->r->high = 0;
		if( res->is_complex ) {
			uip = (t__uint*) mxGetPi( mxp ); 
				//acess the item at index idx
			mb2sn->uint = *(uip + idx);
			res->i->is_double = FALSE;
			res->i->low = mb2sn->uint;
			res->i->high = 0;
		};
	}
}
void sn_matlab_get_mx_item_as_uint64(SN_TYPE(matlab_mx_array_s) arr, 
							SN_TYPE(uint) ro, SN_TYPE(uint) co, SN_TYPE(sn_matlab_complex_s) res) {
	sn_matlab_get_mx_item(arr, ro, co, res);
};
	//free a previosly allocated mxArray
void sn_matlab_free_mx(SN_TYPE(matlab_mx_array_s) arr) {
  //	mxArray* mxp = (mxArray*) arr->mx_ptr;
        mxArray* mxp = (mxArray*) get_mx_ptr(arr);
	if( mxp != 0 ) {
		mxFree( mxp );
		//arr->mx_ptr = 0;
		set_mx_ptr(arr,0);
	};
}

